@@ -1,12 +1,7 @@
-/*
- *	$XConsortium: x2pmp.c,v 1.13 91/07/25 17:56:32 rws Exp $
- */
-
 /* x2pmp.c: Translate xwd window dump format into PMP format for the
  * IBM 3812 PagePrinter.
  */
 #include <stdio.h>
-#include <math.h>
 #include <X11/Xlib.h>
 #include <X11/XWDFile.h>
 #include <X11/Xfuncs.h>
